package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.sprite.Sprite;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * This is similar to the display class in that it requires event messages sent
 * to it. Unlike the display, it has no mechanism for sending messages. Instead,
 * all the event methods return a boolean value indicating if the object's state
 * has changed.
 *
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Comp
{
// *********************
// ***** VARIABLES *****
// *********************
    
    protected boolean _isActive;
    protected boolean _isVisible;
    protected boolean _isSelected;
    protected Rectangle _bounds;
    protected Sprite _sprite;
    protected boolean _changed;
    
    
// ************************
// ***** CONSTRUCTORS *****
// ************************

    /**
     * Default empty component. 16x16 region in top left corner.
    */
    public Comp()
    {
        _sprite = null;
        _bounds = new Rectangle(0,0,16,16);
        _isVisible = true;
        _isActive = true;
        _isSelected = false;
    }
    
    /**
     * Sprite component. Sets component to sprite, using sprites bounds.
     *
     * @param s sprite to use
    */
    public Comp(Sprite s)
    {
        _sprite = s;
        _bounds = s.getBounds();
        _isVisible = s.isVisible();
        _isActive = true;
        _isSelected = false;
    }
    
    /**
     * Sprite component. Sets component to sprite, using adjusted sprites bounds.
     *
     * @param s sprite to use
     * @param x location to move sprite to
     * @param y location to move sprite to
    */
    public Comp(Sprite s, int x, int y)
    {
        _sprite = s;
        s.move(x,y);
        _bounds = s.getBounds();
        _isVisible = s.isVisible();
        _isActive = true;
        _isSelected = false;
    }


// **************************************************
// ***** METHODS - Information and Modification *****
// **************************************************

    /**
     * Moves the component to a new location.
     *
     * @param x new x coordinate of the component
     * @param y new y coordinate of the component
    */
    public void move(int x, int y)
    {
        _bounds.setLocation(x, y);
        _changed = true;
    }
    
    /**
     * @return Leftmost coordinate
     */
    public int getX()
    {
	return _bounds.x;
    }

    /**
     * @return topmost coordinate
     */
    public int getY()
    {
	return _bounds.y;
    }

    /**
     * sets the location and the dimensions of the component.
     *
     * @param x new x coordinate of the component
     * @param y new y coordinate of the component
     * @param w new width of the component
     * @param h new height of the component
    */
    public void setBounds(int x, int y, int w, int h)
    {
        _bounds.setBounds(x, y, w, h);
        _changed = true;
    }
    
    /**
     * sets the location and dimensions of the component to match the provided rectangle
     *
     * @param r new location and dimensions for this component
    */
    public void setBounds(Rectangle r)
    {
        _bounds.setBounds(r);
        _changed = true;
    }
    
    /**
     * @return the bounds of this component
    */
    public Rectangle getBounds()
    {
        return new Rectangle(_bounds);
    }

    /**
     * @return current width of the component
     */
    public int getWidth()
    {
	return _bounds.width;
    }

    /**
     * @return the current height of the component
     */
    public int getHeight()
    {
	return _bounds.height;
    }

    /**
     * Sets the visibility of the component.
     * 
     * @param b set to true to show the component, false to hide it.
    */
    public void setVisible(boolean b)
    {
        _isVisible = b;
        _changed = true;
    }
    
    /**
     * @return true if component is currently visible, false otherwise
    */
    public boolean isVisible()
    {
        return _isVisible;
    }
    
    /**
     * Sets if a component is active or not. A non-active component generally
     * looks different and doesn't respond to events.
     *
     * @param b true if component is to be active, false otherwise
    */
    public void setActive(boolean b)
    {
        _isActive = b;
        _changed = true;
    }
    
    /**
     * @return true if component is active, false otherwise
    */
    public boolean isActive()
    {
        return _isActive;
    }
    
    /**
     * Checks to see if indicated coordinates are within component
     *
     * @param x x coordinate of point to check
     * @param y y coordinate of point to check
     * @return true if point intersects component
    */
    public boolean isIntersecting(int x, int y)
    {
        return _bounds.contains(x, y);
    }
    
    /**
     * Checks to see if indicated coordinates are within component
     *
     * @param p coordinates of point to check
     * @return true if point intersects component
    */
    public boolean isIntersecting(Point p)
    {
        return _bounds.contains(p);
    }
    
    /**
     * Checks to see if indicated rectangle intersects with component
     *
     * @param r rectangle to check
     * @return true if rectangle intersects component
    */
    public boolean isIntersecting(Rectangle r)
    {
        return _bounds.intersects(r);
    }


// ************************************
// ***** METHODS - Event Handling *****
// ************************************

    /**
     * Called when mouse is moved. 
     *
     * @param x x coordinate of mouse pointer
     * @param y y coordinate of mouse pointer
     * @param b this is true if mouse button currently is down
     * @return true if mouse is over component.
    */
    public boolean mouse_move(int x, int y, boolean b)
    {
        if (_isActive)
            return _bounds.contains(x,y);
        else
            return false;
    }
    
    /**
     * Called when mouse has been pressed.
     * 
     * @param x x coordinate of where mouse was clicked
     * @param y y coordinate of where mouse was clicked
     * @return true if mouse went down within this component
    */
    public boolean mouse_down(int x, int y)
    {
        if (_isActive)
        {
            _isSelected = _bounds.contains(x, y);
            return _isSelected;
        }
        else
            return false;
    }
    
    /** 
     * Called when mouse button has been released.
     *
     * @param x x coordinate of where mouse was released
     * @param y y coordinate of where mouse was released
     * @return true if component clicked (pressed and released in component)
    */
    public boolean mouse_up(int x, int y)
    {
        if (_isActive == false)
            return false;
        _changed = _isSelected;
        _isSelected = false;
        if (_changed)
        {
            return _bounds.contains(x, y);
        }
        else
            return false;
    }

    /**
     * Called at a regular interval. Useful for animation.
     *
     * @param ms number of milliseconds that passed since last tick.
     * @return true if state or image changed
    */
    public boolean tick(int ms)
    {
        if (_sprite == null)
            return false;
        boolean temp = _changed;
        temp |= _sprite.isDirty();
        _changed = false;
        return temp;
    }


// *******************************
// ***** METHODS - Rendering *****
// *******************************
    
    /**
     * Draws the component.
     *
     * @param g Graphics Context to draw into
     * @param c Component class that some awt commands need
    */
    public void render(Graphics g, Component c)
    {
        _changed = false;
        if (_sprite != null)
        {
            _sprite.setBounds(_bounds);
            _sprite.setVisible(_isVisible);
            _sprite.setBounding(_isSelected);
            _sprite.render(g, c);
            _sprite.clean();
        }
    }

}
