package com.blazinggames.urcl.ui.sprite;

// Java imports
import java.awt.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * This sprite serves the duel purpose of being your basic image sprite
 * while also having image strip capablilties, which allows it to be used
 * for animation or any other purpose that requires a sprite to be able
 * to switch between different images.
 * <P>
 * Image strips are defined by a width and a height, which is then used to
 * find the clipping for the frame. Essentially we assume that a strip
 * will be broken into rows and columns like a grid. The first frame (0)
 * is the top left, then we proceed to the right for each additional frame,
 * going to the next row when the frame is greater than the number of frames
 * in that row.
 *
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class ImageStripSprite
extends Sprite
{
// *********************
// ***** VARIABLES *****
// *********************

    protected Image _image;
    protected int _width;
    protected int _height;
    protected int _frame;


// ************************
// ***** CONSTRUCTORS *****
// ************************

    /**
     * Standard empty constructor.
     */
    public ImageStripSprite()
    {
        super();
        _image = null;
        _width = _height = _frame = 0;
    }

    /**
     * Single frame image strip. This creates an image strip that consists of the
     * full image provided.
     *
     * @param c component for awt requirements
     * @param i the image
     */
    public ImageStripSprite(Component c, Image i)
    {
        super();
        setImage(c, i);
        setSizeToFrameSize();
    }

    /**
     * Proper image strip.
     *
     * @param c component for awt requirements
     * @param i the image
     * @param w width of each frame
     * @param h height of each frame
     */
    public ImageStripSprite(Component c, Image i, int w, int h)
    {
        super();
        setImage(c, i, w, h);
        setSizeToFrameSize();
    }


// *********************************
// ***** METHODS - From Sprite *****
// *********************************

    /**
     * Draws the sprite on the provided graphics context.
     * Graphics context font and color may be changed!
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     */
    public void render(Graphics g, Component c)
    {
        render(g, c, Sprite.NO_OFFSET);
    }

    /**
     * Same as render, but adjusts the sprites coordinates by adding offset to coordinates.
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     * @param offset point relative to top left that sprite is to be drawn
     */
    public void render(Graphics g, Component c, Point offset)
    {
        if ( (_isVisible == false) || (_image == null) )
            return;
        super.render(g, c, offset);
        int cols = _image.getWidth(c) / _width;
        int sx1 = (_frame % cols) * _width;
        int sy1 = (_frame / cols) * _height;
        int sx2 = sx1 + _width;
        int sy2 = sy1 + _height;
        int dx1 = _bounds.x + offset.x;
        int dy1 = _bounds.y + offset.y;
        int dx2 = dx1 + _bounds.width;
        int dy2 = dy1 + _bounds.height;
        g.drawImage(_image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, c);
    }


// *************************************************************
// ***** METHODS - Additional Information and modification *****
// *************************************************************

    /**
     * Single frame image strip. This creates an image strip that consists of the
     * full image provided. Note that the sprite size is not changed. to change this
     * to match the frame size use the setSizeToFrameSize(); method.
     *
     * @param c component for awt requirements
     * @param i the image
     */
    public void setImage(Component c, Image i)
    {
        _image = i;
        _width = i.getWidth(c);
        _height = i.getHeight(c);
        _frame = 0;
    }

    /**
     * Proper image strip. Note that the sprite size is not changed. to change this
     * to match the frame size use the setSizeToFrameSize(); method.
     *
     * @param c component for awt requirements
     * @param i the image
     * @param w width of each frame
     * @param h height of each frame
     */
    public void setImage(Component c, Image i, int w, int h)
    {
        _image = i;
        _width = w;
        _height = h;
        _frame = 0;
    }

    /**
     * A utility method that makes the sprite the same size as a frame.
     */
    public void setSizeToFrameSize()
    {
        _bounds.setSize(_width, _height);
        invalidate();
    }

    /**
     * Sets the sprite up so it will show the indicated frame.
     *
     * @param n frame that should be shown
     */
    public void setFrame(int n)
    {
        _frame = n;
        invalidate();
    }

    /**
     * @return the current frame that is being displayed
     */
    public int getFrame()
    {
        return _frame;
    }
    
}
